from singer_sdk.pagination import BaseHATEOASPaginator

class FreshservicePaginator(BaseHATEOASPaginator):
    """Gets the next URL from the Links header in the response. This is standard HATEOAS and the Meltano SDK supports it"""
    def get_next_url(self, response):
        next_page_url = response.links.get('next', {}).get('url')
        return next_page_url



"""Stream type classes for tap-freshservice."""

from __future__ import annotations

import typing as t
from pathlib import Path
from urllib.parse import urlencode

import requests
from singer_sdk import typing as th  # JSON Schema typing helpers

from tap_freshservice.client import FreshserviceIncrementalStream

class TicketsStream(FreshserviceIncrementalStream):
    name = "tickets"
    path = "/tickets"
    records_jsonpath="$.tickets[*]"

    def get_url_params(self, context: dict | None, next_page_token) -> dict[str, t.Any] | str:
        parent_params = super().get_url_params(context, next_page_token)
        params = {"include": "stats"}
        params.update(parent_params)
        return params

    schema = th.PropertiesList(
        th.Property("subject", th.StringType),
        th.Property("group_id", th.IntegerType),
        th.Property("department_id", th.IntegerType),
        th.Property("category", th.StringType),
        th.Property("sub_category", th.StringType),
        th.Property("item_category", th.StringType),
        th.Property("requester_id", th.IntegerType),
        th.Property("responder_id", th.IntegerType),
        th.Property("due_by", th.StringType),
        th.Property("fr_escalated", th.BooleanType),
        th.Property("deleted", th.BooleanType),
        th.Property("spam", th.BooleanType),
        th.Property("email_config_id", th.StringType),
        th.Property("fwd_emails", th.ArrayType(th.StringType)),
        th.Property("reply_cc_emails", th.ArrayType(th.StringType)),
        th.Property("cc_emails", th.ArrayType(th.StringType)),
        th.Property("is_escalated", th.BooleanType),
        th.Property("fr_due_by", th.StringType),
        th.Property("id", th.IntegerType),
        th.Property("priority", th.IntegerType),
        th.Property("status", th.IntegerType),
        th.Property("source", th.IntegerType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("requested_for_id", th.IntegerType),
        th.Property("to_emails", th.StringType),
        th.Property("type", th.StringType),
        th.Property("description", th.StringType),
        th.Property("description_text", th.StringType),
        th.Property("custom_fields", th.ObjectType(
            th.Property("estimate", th.StringType),
            th.Property("pending_reason", th.StringType),
            th.Property("quote_hrs", th.StringType),
            th.Property("definition_of_done", th.StringType)
        )),
        th.Property("stats", th.ObjectType(
            th.Property("created_at", th.DateTimeType),
            th.Property("updated_at", th.DateTimeType),
            th.Property("ticket_id", th.IntegerType),
            th.Property("opened_at", th.DateTimeType),
            th.Property("group_escalated", th.BooleanType),
            th.Property("inbound_count", th.IntegerType),
            th.Property("status_updated_at", th.DateTimeType),
            th.Property("outbound_count", th.IntegerType),
            th.Property("pending_since", th.StringType),
            th.Property("resolved_at", th.DateTimeType),
            th.Property("closed_at", th.DateTimeType),
            th.Property("first_assigned_at", th.DateTimeType),
            th.Property("assigned_at", th.DateTimeType),
            th.Property("agent_responded_at", th.DateTimeType),
            th.Property("requester_responded_at", th.DateTimeType),
            th.Property("first_responded_at", th.DateTimeType),
            th.Property("first_resp_time_in_secs", th.IntegerType),
            th.Property("resolution_time_in_secs", th.IntegerType)
        ))
    ).to_dict()




"""REST client handling, including FreshserviceStream base class."""

from __future__ import annotations

from typing import Any, Iterable
from urllib.parse import parse_qsl

import requests
from singer_sdk.authenticators import BasicAuthenticator
from singer_sdk.helpers.jsonpath import extract_jsonpath
from singer_sdk.pagination import BaseAPIPaginator  # noqa: TCH002
from singer_sdk.streams import RESTStream

from tap_freshservice.paginator import FreshservicePaginator

class FreshserviceStream(RESTStream):
    """Freshservice stream class."""

    @property
    def url_base(self) -> str:
        """Return the API URL root, configurable via tap settings."""
        return self.config.get('url_base')  

    @property
    def authenticator(self) -> BasicAuthenticator:
        """Return a new authenticator object.

        Returns:
            An authenticator instance.
        """
        return BasicAuthenticator.create_for_stream(
            self,
            username=self.config.get("api_key"),
            password="X",
        )

    @property
    def http_headers(self) -> dict:
        """Return the http headers needed.

        Returns:
            A dictionary of HTTP headers.
        """
        headers = {}
        if "user_agent" in self.config:
            headers["User-Agent"] = self.config.get("user_agent")
        # If not using an authenticator, you may also provide inline auth headers:
        # headers["Private-Token"] = self.config.get("auth_token")  # noqa: ERA001
        return headers

    def get_new_paginator(self) -> BaseAPIPaginator:
        """Create a new pagination helper instance.
        Returns:
            A pagination helper instance.
        """
        return FreshservicePaginator()

    def get_url_params(
        self,
        context: dict | None,  # noqa: ARG002
        next_page_token: Any | None,  # noqa: ANN401
    ) -> dict[str, Any]:
        """Return a dictionary of values to be used in URL parameterization.

        Args:
            context: The stream context.
            next_page_token: The next page index or value.

        Returns:
            A dictionary of URL query parameters.
        """
        params: dict = {}
        if next_page_token:
            params["page"] = dict(parse_qsl(next_page_token.query)).get('page')

        return params



class FreshserviceIncrementalStream(FreshserviceStream):
    replication_key = "updated_at"
    is_sorted = True

    def get_url_params(
        self,
        context: dict | None,  # noqa: ARG002
        next_page_token: Any | None,  # noqa: ANN401
    ) -> dict[str, Any]:
        """Return a dictionary of values to be used in URL parameterization.

        Args:
            context: The stream context.
            next_page_token: The next page index or value.

        Returns:
            A dictionary of URL query parameters.
        """
        params: dict = {}
        if self.replication_key:
            params["order_type"] = "asc"
            params["order_by"] = self.replication_key
            starting_timestamp = self.get_starting_timestamp(context)

            # By default only tickets that have been created within the past 30 days will be returned. For older tickets, use the updated_since filter.
            if starting_timestamp:
                params["updated_since"] = starting_timestamp.isoformat()
            elif self.config.get('updated_since') is not None:
                params["updated_since"] = self.config.get('updated_since')

        params.update(super().get_url_params(context, next_page_token))
        return params


"""Freshservice tap class."""

from __future__ import annotations

from singer_sdk import Tap
from singer_sdk import typing as th  # JSON schema typing helpers

from tap_freshservice.streams import (groups, tickets)


class TapFreshservice(Tap):
    """Freshservice tap class."""

    name = "tap-freshservice"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "api_key",
            th.StringType,
            required=True,
            secret=True,  # Flag config as protected.
            description="The Freshservice API key",
        ),
        th.Property(
            "updated_since",
            th.DateTimeType,
            description="The earliest record date to sync. You probably need this! The Freshservice API only returns items less than 30 days old. To override this, you must include an 'updated_since' value in the URL querystring. Providing a value here will ensure this value is used if there is no state (i.e. for a full refresh). ",
            default="2000-01-01T00:00:00Z"
        ),
        th.Property(
            "base_url",
            th.StringType,
            required=True,
            default="https://<replace with your org>.freshservice.com/api/v2",
            description="The url for the Freshservice API",
        ),
    ).to_dict()

    def discover_streams(self) -> list[tickets.FreshserviceStream]:
        """Return a list of discovered streams.

        Returns:
            A list of discovered streams.
        """
        return [
            tickets.TicketsStream(self),
            groups.GroupsStream(self),
        ]


if __name__ == "__main__":
    TapFreshservice.cli()



"""Stream type classes for tap-freshservice."""
from singer_sdk import typing as th  # JSON Schema typing helpers

from tap_freshservice.client import FreshserviceStream

class GroupsStream(FreshserviceStream):
    name = "groups"
    path = "/groups"
    records_jsonpath="$.groups[*]"

    def get_url(self, context: dict):
        url = super().get_url(context)
        return url
    
    def build_prepared_request(self, *args, **kwargs):
        req = super().build_prepared_request(*args, **kwargs)
        return req

    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("description", th.StringType),
        th.Property("escalate_to", th.IntegerType),
        th.Property("unassigned_for", th.StringType),
        th.Property("business_hours_id", th.IntegerType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("auto_ticket_assign", th.BooleanType),
        th.Property("ocs_schedule_id", th.IntegerType),
        th.Property("members", th.ArrayType(th.IntegerType)),
    ).to_dict()

